@@ -11,7 +11,7 @@ AUTHORS:
 
 - Florent Hivert (2009/02/06): doc improvements + new methods
 """
-#*****************************************************************************
+# ****************************************************************************
 #       Copyright (C) 2007 Mike Hansen <mhansen@gmail.com>,
 #                     2014 Vincent Delecroix <20100.delecroix@gmail.com>,
 #
@@ -24,8 +24,8 @@ AUTHORS:
 #
 #  The full text of the GPL is available at:
 #
-#                  http://www.gnu.org/licenses/
-#*****************************************************************************
+#                  https://www.gnu.org/licenses/
+# ****************************************************************************
 from __future__ import print_function, absolute_import
 
 import six
@@ -850,6 +850,7 @@ class Subsets_sk(Subsets_s):
         """
         return self.unrank(self.cardinality() // 2)
 
+
 def dict_to_list(d):
     r"""
     Return a list whose elements are the elements of i of d repeated with
@@ -862,27 +863,37 @@ def dict_to_list(d):
         ['a', 'b', 'b', 'b']
     """
     l = []
-    for i,j in six.iteritems(d):
-        l.extend([i]*j)
+    for i, j in six.iteritems(d):
+        l.extend([i] * j)
     return l
 
+
 def list_to_dict(l):
     r"""
-    Return a dictionary whose keys are the elements of l and values are the
-    multiplicity they appear in l.
+    Return a dictionary of multiplicities and the list of its keys.
+
+    INPUT:
+
+    a list ``l`` with possibly repeated elements
+
+    The keys are the elements of ``l`` (in the same order in which they appear)
+    and values are the multiplicities of each element in ``l``.
 
     EXAMPLES::
 
         sage: from sage.combinat.subset import list_to_dict
         sage: list_to_dict(['a', 'b', 'b', 'b'])
-        {'a': 1, 'b': 3}
+        ({'a': 1, 'b': 3}, ['a', 'b'])
     """
     d = {}
+    keys = []
     for elt in l:
         if elt not in d:
             d[elt] = 0
+            keys.append(elt)
         d[elt] += 1
-    return d
+    return d, keys
+
 
 class SubMultiset_s(Parent):
     """
@@ -917,7 +928,7 @@ class SubMultiset_s(Parent):
 
     def __init__(self, s):
         """
-        Constructs the combinatorial class of the sub multisets of s.
+        Construct the combinatorial class of the sub multisets of s.
 
         EXAMPLES::
 
@@ -925,12 +936,23 @@ class SubMultiset_s(Parent):
             sage: Subsets([1,2,3,3], submultiset=True).cardinality()
             12
             sage: TestSuite(S).run()
+
+        TESTS::
+
+            sage: from sage.combinat.subset import SubMultiset_s
+            sage: T = SubMultiset_s({'a':2,'b':1,'c':2})
+            sage: T.cardinality()
+            18
         """
         Parent.__init__(self, category=FiniteEnumeratedSets())
 
-        self._d = s
-        if not isinstance(s, dict):
-            self._d = list_to_dict(s)
+        if isinstance(s, dict):
+            self._d = s
+            self._l = dict_to_list(s)
+            self._keys = list(self._d)
+        else:
+            self._l = s
+            self._d, self._keys = list_to_dict(s)
 
     def _repr_(self):
         """
@@ -939,7 +961,7 @@ class SubMultiset_s(Parent):
             sage: S = Subsets([1, 2, 2, 3], submultiset=True); S #indirect doctest
             SubMultiset of [1, 2, 2, 3]
         """
-        return "SubMultiset of {}".format(dict_to_list(self._d))
+        return "SubMultiset of {}".format(self._l)
 
     def __eq__(self, other):
         r"""
@@ -997,7 +1019,7 @@ class SubMultiset_s(Parent):
 
     def cardinality(self):
         r"""
-        Return the cardinality of self
+        Return the cardinality of ``self``.
 
         EXAMPLES::
 
@@ -1020,11 +1042,11 @@ class SubMultiset_s(Parent):
             24
         """
         from sage.all import prod
-        return Integer(prod(k+1 for k in self._d.values()))
+        return Integer(prod(k + 1 for k in self._d.values()))
 
     def random_element(self):
         r"""
-        Return a random element of self with uniform law
+        Return a random element of ``self`` with uniform law.
 
         EXAMPLES::
 
@@ -1033,14 +1055,14 @@ class SubMultiset_s(Parent):
             [2]
         """
         l = []
-        for i in self._d:
-            l.extend([i]*rnd.randint(0,self._d[i]))
+        for i, di in self._d.items():
+            l.extend([i] * rnd.randint(0, di))
         return l
 
-    def generating_serie(self,variable='x'):
+    def generating_serie(self, variable='x'):
         r"""
-        Return the serie (here a polynom) associated to the counting of the
-        element of self weighted by the number of element they contain.
+        Return the polynomial associated to the counting of the
+        elements of ``self`` weighted by the number of element they contain.
 
         EXAMPLES::
 
@@ -1057,15 +1079,16 @@ class SubMultiset_s(Parent):
             sage: sum(S.generating_serie())
             72
         """
-        from sage.all import prod
         R = ZZ[variable]
-        return prod(R([1]*(n+1)) for n in self._d.values())
+        return R.prod(R([1] * (n + 1)) for n in self._d.values())
 
     def __iter__(self):
         """
-        Iterates through the subsets of ``self``.  Note that each subset is
-        represented by a list of its elements rather than a set since we can
-        have multiplicities (no multiset data structure yet in sage).
+        Iterate through the subsets of ``self``.
+
+        Note that each subset is represented by a list of its elements
+        rather than a set since we can have multiplicities (no
+        multiset data structure yet in sage).
 
         EXAMPLES::
 
@@ -1083,10 +1106,9 @@ class SubMultiset_s(Parent):
              [1, 2, 3],
              [2, 2, 3],
              [1, 2, 2, 3]]
-
         """
-        for k in range(sum(self._d.values())+1):
-            for s in SubMultiset_sk(self._d, k):
+        for k in range(len(self._l) + 1):
+            for s in SubMultiset_sk(self._l, k):
                 yield s
 
     def __call__(self, el):
@@ -1154,7 +1176,6 @@ class SubMultiset_sk(SubMultiset_s):
             sage: TestSuite(S).run()
         """
         SubMultiset_s.__init__(self, s)
-        self._l = dict_to_list(self._d)
         self._k = k
 
     def __eq__(self, other):
@@ -1170,10 +1191,10 @@ class SubMultiset_sk(SubMultiset_s):
             return False
         return self._d == other._d and self._k == other._k
 
-    def generating_serie(self,variable='x'):
+    def generating_serie(self, variable='x'):
         r"""
-        Return the serie (this case a polynom) associated to the counting of the
-        element of self weighted by the number of element they contains
+        Return the polynomial associated to the counting of the
+        elements of ``self`` weighted by the number of elements they contains
 
         EXAMPLES::
 
@@ -1196,7 +1217,7 @@ class SubMultiset_sk(SubMultiset_s):
 
     def cardinality(self):
         r"""
-        Return the cardinality of self
+        Return the cardinality of ``self``.
 
         EXAMPLES::
 
@@ -1261,8 +1282,10 @@ class SubMultiset_sk(SubMultiset_s):
 
     def __iter__(self):
         """
-        Iterates through the subsets of size ``self._k`` of the multiset
-        ``self._s``. Note that each subset is represented by a list of the
+        Iterate through the subsets of size ``self._k`` of the multiset
+        ``self._s``.
+
+        Note that each subset is represented by a list of the
         elements rather than a set since we can have multiplicities (no
         multiset data structure yet in sage).
 
@@ -1273,10 +1296,12 @@ class SubMultiset_sk(SubMultiset_s):
             [[1, 2], [1, 3], [2, 2], [2, 3]]
         """
         from sage.combinat.integer_vector import IntegerVectors
-        elts = list(self._d)
-        for iv in IntegerVectors(self._k, len(self._d), outer=list(self._d.values())):
+        elts = self._keys
+        for iv in IntegerVectors(self._k, len(self._d),
+                                 outer=list(self._d.values())):
             yield sum([[elts[i]] * iv[i] for i in range(len(iv))], [])
 
+
 class SubsetsSorted(Subsets_s):
     """
     Lightweight class of all subsets of some set `S`, with each
